@@ -7,25 +7,13 @@ use Pod::Usage;
 use Getopt::Long;
 use FindBin qw($Bin);
 use Statistics::Lite qw(mean median sum stddevp min max);
-use Bio::ToolBox::data_helper qw(
-	find_column_index
-	format_with_commas
-	splice_data_structure
-);
+use Bio::ToolBox::Data;
 use Bio::ToolBox::db_helper qw(
 	open_db_connection
 	verify_or_request_feature_types
-	get_new_feature_list
-	get_feature
-	get_region_dataset_hash
-	get_chromo_region_score
 	check_dataset_for_rpm_support
 );
-use Bio::ToolBox::file_helper qw(
-	load_tim_data_file
-	write_tim_data_file
-	write_summary_data
-);
+use Bio::ToolBox::utility;
 my $parallel;
 eval {
 	# check for parallel support
@@ -33,14 +21,14 @@ eval {
 	$parallel = 1;
 };
 use constant LOG2 => log(2);
-use constant DATASET_HASH_LIMIT => 3000;
+use constant DATASET_HASH_LIMIT => 5001;
 		# This constant determines the maximum size of the dataset hash to be 
 		# returned from the get_region_dataset_hash(). To increase performance, 
 		# the program normally queries the database once for each feature or 
 		# region, and a hash returned with potentially a score for each basepair. 
 		# This may become unwieldy for very large regions, which may be better 
 		# served by separate database queries for each window.
-my $VERSION = '1.18';
+my $VERSION = '1.20';
 
 print "\n A script to collect windowed data flanking a relative position of a feature\n\n";
   
@@ -140,63 +128,49 @@ my $start_time = time;
 
 
 ## Generate or load the input dataset
-my $main_data_ref; # a reference to the main data structure
+my $Data;
 if ($infile) {
-	# load the gene dataset from existing file
-	print " Loading feature set from file $infile....\n";
-	$main_data_ref = load_tim_data_file($infile);
-} 
+	$Data = Bio::ToolBox::Data->new(file => $infile) or 
+		die " unable to load input file '$infile'\n";
+	printf "  Loaded %s '%s' features.\n", 
+		format_with_commas( $Data->last_row ), $Data->feature;
+	
+	# update main database as necessary
+	if ($main_database) {
+		if ($main_database ne $Data->database) {
+			# update with new database
+			printf " updating main database name from '%s' to '%s'\n", 
+				$Data->database, $main_database;
+			print "   Re-run without --db option if you do not want this to happen\n";
+			$Data->database($main_database);
+		}
+	}
+	else {
+		$main_database = $Data->database;
+	}
+}
 else {
-	# we will start a new file with a new dataset
-	$main_data_ref = get_new_feature_list(
-			'db'       => $main_database,
-			'features' => $feature,
-	);
+	# generate a new file
+	print " Generating a new feature list from database '$main_database'...\n";
+	$Data = Bio::ToolBox::Data->new(
+		db      => $main_database,
+		feature => $feature,
+	) or die " unable to generate new feature list\n";
 }
-unless ($main_data_ref) {
-	# check for data
-	die " No data loaded! Nothing to do!\n";
-}
-
-# update program name
-unless ($main_data_ref->{'program'} eq $0) {
-	$main_data_ref->{'program'} = $0;
-}
-
-# simple reference to the data table
-my $data_table_ref = $main_data_ref->{'data_table'};
 
 # the number of columns already in the data array
-my $startcolumn = $main_data_ref->{'number_columns'}; 
-
-
+my $startcolumn = $Data->number_columns; 
 
+# make sure data table supports avoid option
+if ($avoid) {
+	unless ($Data->feature_type eq 'named' and defined $Data->type_column) {
+		warn " avoid option not supported with current Data table. Disabling\n";
+		$avoid = 0;
+	}
+}
 
 
 ## Prepare to collect data
-
-# Open main database connection
-unless ($main_database) {
-	# define the database 
-	if ( $main_data_ref->{'db'} ) {
-		# from the input file if possible
-		$main_database = $main_data_ref->{'db'};
-	}
-	elsif ($dataset) {
-		# or use the dataset if possible
-		$main_database = $dataset;
-	}
-	else {
-		die " You must define a database or dataset!\n" . 
-			" Use --help for more information\n";
-	}
-}
-my $mdb = open_db_connection($main_database);
-unless ($mdb) {
-	die " You must define a database or dataset!\n" . 
-		" Use --help for more information\n";
-}
-
 # Open data database
 my $ddb;
 if (defined $data_database) {
@@ -204,14 +178,10 @@ if (defined $data_database) {
 	$ddb = open_db_connection($data_database) or 
 		die "unable to establish data database connection to $data_database!\n";
 }
-else {
-	# reuse the main database connection
-	$ddb = $mdb;
-}
 
 # Check the dataset
 $dataset = verify_or_request_feature_types(
-	'db'      => $ddb,
+	'db'      => $ddb || $Data->database,
 	'feature' => $dataset,
 	'single'  => 1,
 	'prompt'  => " Enter the number of the feature or dataset from which to" . 
@@ -237,7 +207,7 @@ if ($method eq 'rpm') {
 
 # check whether it is worth doing parallel execution
 if ($cpu > 1) {
-	while ($cpu > 1 and $main_data_ref->{'last_row'}/$cpu < 100) {
+	while ($cpu > 1 and $Data->last_row / $cpu < 100) {
 		# I figure we need at least 100 lines in each fork split to make 
 		# it worthwhile to do the split, otherwise, reduce the number of 
 		# splits to something more worthwhile
@@ -278,7 +248,6 @@ sub check_defaults {
 			die " You must define an output filename !\n Use --help for more information\n";
 		}
 	}
-	$outfile =~ s/\.txt$//; # strip extension, we'll add it later
 
 	unless ($feature or $infile) {
 		die " You must define a feature or use an input file!\n Use --help for more information\n";
@@ -387,7 +356,7 @@ sub check_defaults {
 	else {
 		# disable cores
 		print " disabling parallel CPU execution, no support present\n" if $cpu;
-		$cpu = 0;
+		$cpu = 1;
 	}
 }
 
@@ -406,17 +375,13 @@ sub parallel_execution {
 		#### In child ####
 	
 		# splice the data structure
-		splice_data_structure($main_data_ref, $i, $cpu);
+		$Data->splice_data($i, $cpu);
 		
 		# re-open database objects to make them clone safe
 		# pass second true to avoid cached database objects
-		$mdb = open_db_connection($main_database, 1);
 		if ($data_database) {
 			$ddb = open_db_connection($data_database, 1);
 		}
-		else {
-			$ddb = $mdb;
-		}
 		
 		# Collect the data
 		map_relative_data();
@@ -433,8 +398,7 @@ sub parallel_execution {
 		}
 		
 		# write out result
-		my $success = write_tim_data_file(
-			'data'     => $main_data_ref,
+		my $success = $Data->save(
 			'filename' => "$child_base_name.$i",
 			'gz'       => 0, # faster to write without compression
 		);
@@ -466,14 +430,13 @@ sub parallel_execution {
 	# generate summary file
 	if ($sum) {
 		# reopen the combined file
-		my $data = load_tim_data_file($outfile);
-		unless ($data) {
+		my $Data2 = Bio::ToolBox::Data->new(file => $outfile);
+		unless ($Data2) {
 			warn " cannot re-open $outfile to generate summary file!\n";
 			return;
 		}
 		print " Generating final summed data....\n";
-		my $sumfile = write_summary_data(
-			'data'        => $data,
+		my $sumfile = $Data2->summary_file(
 			# it will automatically define a new output name
 			'startcolumn' => $startcolumn,
 			'dataset'     => $dataset,
@@ -512,8 +475,7 @@ sub single_execution {
 	# an average across all features at each position suitable for plotting
 	if ($sum) {
 		print " Generating final summed data....\n";
-		my $sumfile = write_summary_data(
-			'data'        => $main_data_ref,
+		my $sumfile = $Data->summary_file(
 			'filename'    => $outfile,
 			'startcolumn' => $startcolumn,
 			'dataset'     => $dataset,
@@ -529,8 +491,7 @@ sub single_execution {
 
 
 	## Output the data
-	my $written_file = write_tim_data_file(
-		'data'     => $main_data_ref,
+	my $written_file = $Data->save(
 		'filename' => $outfile,
 		'gz'       => $gz,
 	);
@@ -602,49 +563,41 @@ sub prepare_window_datasets {
 			}
 		}
 		
-		# set the new index value, which is equivalent to the number of columns
-		my $new_index = $main_data_ref->{'number_columns'};
-		
 		# the new name
 		my $new_name = $start . '..' . $stop;
 		
+		# add new column
+		my $new_index = $Data->add_column($new_name);
+		
 		# set the metadata
-		my %metadata = (
-			'name'        => $new_name,
-			'index'       => $new_index,
-			'start'       => $start,
-			'stop'        => $stop,
-			'window'      => $win,
-			'log2'        => $log,
-			'dataset'     => $dataset,
-			'method'      => $method,
-			'value'       => $value_type,
-		);
+		$Data->metadata($new_index, 'start' , $start);
+		$Data->metadata($new_index, 'stop' , $stop);
+		$Data->metadata($new_index, 'window' , $win);
+		$Data->metadata($new_index, 'log2' , $log);
+		$Data->metadata($new_index, 'dataset' , $dataset);
+		$Data->metadata($new_index, 'method' , $method);
+		$Data->metadata($new_index, 'value' , $value_type);
 		if ($position == 5) {
-			$metadata{'relative_position'} = '5prime_end';
+			$Data->metadata($new_index, 'relative_position', '5prime_end');
 		}
 		elsif ($position == 3) {
-			$metadata{'relative_position'} = '3prime_end';
+			$Data->metadata($new_index, 'relative_position', '3prime_end');
 		}
 		else { # midpoint
-			$metadata{'relative_position'} = 'center';
+			$Data->metadata($new_index, 'relative_position', 'center');
 		}
 		if ($set_strand) {
-			$metadata{'strand_implied'} = 1;
+			$Data->metadata($new_index, 'strand_implied', 1);
 		}
 		if ($strand_sense =~ /sense/) {
-			$metadata{'strand'} = $strand_sense;
+			$Data->metadata($new_index, 'strand',  $strand_sense);
 		}
 		if ($data_database) {
-			$metadata{'db'} = $data_database;
+			$Data->metadata($new_index, 'db', $data_database);
+		}
+		if ($avoid) {
+			$Data->metadata($new_index, 'avoid', 1);
 		}
-		$main_data_ref->{$new_index} = \%metadata;
-		
-		# set the column header
-		$data_table_ref->[0][$new_index] = $new_name;
-		
-		# update number of columns
-		$main_data_ref->{'number_columns'} += 1;
 	}
 	
 	return ($startingpoint, $endingpoint);
@@ -660,23 +613,6 @@ sub map_relative_data {
 	my ($starting_point, $ending_point) = prepare_window_datasets();
 	
 	
-	
-	# Identify columns for feature identification
-	my $name   = find_column_index($main_data_ref, '^name');
-	
-	my $type   = find_column_index($main_data_ref, '^type|class');
-	
-	my $id     = find_column_index($main_data_ref, '^primary_id');
-	
-	my $chromo = find_column_index($main_data_ref, '^chr|seq|ref|ref.?seq');
-	
-	my $start  = find_column_index($main_data_ref, '^start|position');
-	
-	my $stop   = find_column_index($main_data_ref, '^stop|end');
-	
-	my $strand = find_column_index($main_data_ref, '^strand');
-	
-	
 	# determine long data collection for very large regions
 	if ($ending_point - $starting_point > DATASET_HASH_LIMIT) {
 		# This could potentially create performance issues where returned hashes   
@@ -685,94 +621,46 @@ sub map_relative_data {
 		$long_data = 1;
 	}
 	
-	
-	
 	# Select the appropriate method for data collection
-	if (
-		defined $start and
-		defined $stop  and
-		defined $chromo and
-		not $long_data
-	) {
+	if ($Data->feature_type eq 'coordinate' and not $long_data) {
 		# mapping point data features using genome segments
-		map_relative_data_for_regions(
-			$starting_point, $ending_point, $chromo, $start, $stop, $strand);
+		map_relative_data_for_regions($starting_point, $ending_point);
 	}
-	
-	elsif (
-		defined $start and
-		defined $stop  and
-		defined $chromo and
-		$long_data
-	) {
+	elsif ($Data->feature_type eq 'coordinate' and $long_data) {
 		# mapping long data features using genome segments
-		map_relative_long_data_for_regions($chromo, $start, $stop, $strand);
+		map_relative_long_data_for_regions();
 	}
-	
-	elsif (
-		(defined $id or 
-			( defined $name and defined $type ) 
-		) and 
-		not $long_data
-	) {
+	elsif ($Data->feature_type eq 'named' and not $long_data) {
 		# mapping point data features using named features
-		map_relative_data_for_features(
-			$starting_point, $ending_point, $name, $type, $id, $strand);
+		map_relative_data_for_features($starting_point, $ending_point);
 	}
-	
-	elsif (
-		(defined $id or 
-			( defined $name and defined $type ) 
-		) and 
-		$long_data
-	) {
+	elsif ($Data->feature_type eq 'named' and $long_data) {
 		# mapping long data features using named features
-		map_relative_long_data_for_features($name, $type, $id, $strand);
+		map_relative_long_data_for_features();
 	}
-	
 	else {
 		die " Unable to identify columns with feature identifiers!\n" .
 			" File must have Primary_ID or Name and Type, or Chromo, Start, Stop columns\n";
 	}
-	
 }
 
 
 sub map_relative_data_for_features {
+	my ($starting_point, $ending_point) = @_;
 	
-	# Get the feature indices
-	my (
-		$starting_point, 
-		$ending_point, 
-		$name_index, 
-		$type_index, 
-		$id_index,
-		$strand_index
-	) = @_;
-	
-	
-	### Collect the data
-	for my $row (1..$main_data_ref->{'last_row'}) {
-		
-		# collect the region scores
-		my %regionscores = get_region_dataset_hash(
-				'db'          => $mdb,
-				'ddb'         => $ddb,
-				'dataset'     => $dataset,
-				'name'        => defined $name_index ? 
-									$data_table_ref->[$row][$name_index] : undef,
-				'type'        => defined $type_index ? 
-									$data_table_ref->[$row][$type_index] : undef,
-				'id'          => defined $id_index ?
-									$data_table_ref->[$row][$id_index] : undef,
-				'start'       => $starting_point,
-				'stop'        => $ending_point,
-				'position'    => $position,
-				'value'       => $value_type,
-				'stranded'    => $strand_sense,
-				'strand'      => $set_strand ? 
-								$data_table_ref->[$row][$strand_index] : undef, 
-				'avoid'       => $avoid,
+	# Collect the data
+	my $stream = $Data->row_stream;
+	while (my $row = $stream->next_row) {
+		my %regionscores = $row->get_position_scores(
+			'ddb'         => $ddb,
+			'dataset'     => $dataset,
+			'start'       => $starting_point,
+			'stop'        => $ending_point,
+			'position'    => $position,
+			'value'       => $value_type,
+			'stranded'    => $strand_sense,
+			'strand'      => $set_strand ? $row->strand : undef, 
+			'avoid'       => $avoid,
 		);
 		
 		# record the scores
@@ -784,120 +672,42 @@ sub map_relative_data_for_features {
 
 sub map_relative_long_data_for_features {
 	
-	# Get the feature indices
-	my (
-		$name_index, 
-		$type_index, 
-		$id_index,
-		$strand_index
-	) = @_;
-	
-	
-	### Collect the data
-	for my $row (1..$main_data_ref->{'last_row'}) {
+	# Collect the data
+	my $stream = $Data->row_stream;
+	while (my $row = $stream->next_row) {
 		
 		# get feature from the database
-		my $feature = get_feature(
-			'db'    => $mdb,
-			'id'    => defined $id_index ? 
-				$data_table_ref->[$row][$id_index] : undef,
-			'name'  => defined $name_index ? 
-				$data_table_ref->[$row][$name_index] : undef,
-			'type'  => defined $type_index ? 
-				$data_table_ref->[$row][$type_index] : undef,
-		);
-		
-		# check that we have a feature
+		my $feature = $row->feature;
 		unless ($feature) {
-			
 			# record a null values
-			for (
-				my $column = $startcolumn; 
-				$column < $main_data_ref->{'number_columns'}; 
-				$column++
-			) {
-				$data_table_ref->[$row][$column] = '.';
+			for (my $c = $startcolumn; $c < $Data->number_columns; $c++) {
+				$row->value($c, '.');
 			}
-			
-			# move on
 			next;
 		}
 		
-		
 		# Collect the scores for each window
 		collect_long_data_window_scores(
 			$row,
 			$feature->seq_id,
 			$feature->start,
 			$feature->end,
-			$set_strand ? 
-				$data_table_ref->[$row][$strand_index] : $feature->strand
+			$set_strand ? $row->strand : $feature->strand,
 		);
+		
+		if ($avoid) {
+			warn " avoid option is currently not supported with long data collection! Disabling!\n";
+			$avoid = 0;
+		}
 	}
 }
 
 sub map_relative_data_for_regions {
+	my ($starting_point, $ending_point) = @_;
 	
-	# Get the feature indices
-	my (
-		$starting_point, 
-		$ending_point, 
-		$chr_index, 
-		$start_index, 
-		$stop_index, 
-		$strand_index
-	) = @_;
-	
-	
-	### Collect the data
-	for my $row (1..$main_data_ref->{'last_row'}) {
-		
-		# collect the given coordinates from the data table
-		my ($fstart, $fstop, $strand);
-		if (
-			$data_table_ref->[$row][$start_index] <= 
-			$data_table_ref->[$row][$stop_index]
-		) {
-			# proper orientation
-			$fstart = $data_table_ref->[$row][$start_index];
-			$fstop  = $data_table_ref->[$row][$stop_index];
-			
-			# set the strand if not defined
-			unless (defined $strand_index) {
-				$strand = 1;
-			}
-		}
-		else {
-			# not a proper orientation of values
-			# assume the user really meant this
-			$fstart = $data_table_ref->[$row][$stop_index];
-			$fstop  = $data_table_ref->[$row][$start_index];
-			
-			# set the strand if not defined
-			unless (defined $strand_index) {
-				$strand = -1;
-			}
-		}
-			
-		# determine strand
-		if (defined $strand_index) {
-			# the strand in the data table could be any one of several 
-			# characters denoting strand
-			if ($data_table_ref->[$row][$strand_index] =~ m/^[1|f|w|\+]/i) {
-				$strand = 1;
-			}
-			elsif ($data_table_ref->[$row][$strand_index] =~ m/^[r|c|\-]/i) {
-				$strand = -1;
-			}
-			else {
-				$strand = 0;
-			}
-		}
-		else {
-			$strand = 0 unless defined $strand;
-		}
-		
-		
+	# Collect the data
+	my $stream = $Data->row_stream;
+	while (my $row = $stream->next_row) {
 		
 		# calculate new coordinates based on relative adjustments
 			# this is a little tricky, because we're working with absolute 
@@ -905,37 +715,37 @@ sub map_relative_data_for_regions {
 			# the appropriate conversions
 		my ($start, $stop, $region_start);
 		
-		if ($strand >= 0 and $position == 5) {
+		if ($row->strand >= 0 and $position == 5) {
 			# 5' end of forward strand
-			$region_start = $fstart;
-			$start = $fstart + $starting_point;
-			$stop  = $fstart + $ending_point;
+			$region_start = $row->start;
+			$start = $row->start + $starting_point;
+			$stop  = $row->start + $ending_point;
 		}
 		
-		elsif ($strand == -1 and $position == 5) {
+		elsif ($row->strand == -1 and $position == 5) {
 			# 5' end of reverse strand
-			$region_start = $fstop;
-			$start = $fstop - $ending_point;
-			$stop  = $fstop - $starting_point;
+			$region_start = $row->stop;
+			$start = $row->stop - $ending_point;
+			$stop  = $row->stop - $starting_point;
 		}
 		
-		elsif ($strand >= 0 and $position == 3) {
+		elsif ($row->strand >= 0 and $position == 3) {
 			# 3' end of forward strand
-			$region_start = $fstop;
-			$start = $fstop + $starting_point;
-			$stop  = $fstop + $ending_point;
+			$region_start = $row->stop;
+			$start = $row->stop + $starting_point;
+			$stop  = $row->stop + $ending_point;
 		}
 		
-		elsif ($strand == -1 and $position == 3) {
+		elsif ($row->strand == -1 and $position == 3) {
 			# 3' end of reverse strand
-			$region_start = $fstart;
-			$start = $fstart - $ending_point;
-			$stop  = $fstart - $starting_point;
+			$region_start = $row->start;
+			$start = $row->start - $ending_point;
+			$stop  = $row->start - $starting_point;
 		}
 		
 		elsif ($position == 4) {
 			# midpoint regardless of strand
-			$region_start = int( ( ($fstop - $fstart) / 2) + 0.5);
+			$region_start = int( ( ($row->stop + $row->start) / 2) + 0.5);
 			$start = $region_start + $starting_point;
 			$stop  = $region_start + $ending_point;
 		}
@@ -943,17 +753,15 @@ sub map_relative_data_for_regions {
 		else {
 			# something happened
 			die " programming error!? feature " . 
-				" at data row $row\n";
+				" at data row $row->row_index\n";
 		}
 		
 		# collect the region scores
-		my %regionscores = get_region_dataset_hash(
+		my %regionscores = $row->get_position_scores(
 				'db'          => $ddb,
 				'dataset'     => $dataset,
-				'chromo'      => $data_table_ref->[$row][$chr_index],
 				'start'       => $start,
 				'stop'        => $stop,
-				'strand'      => $strand,
 				'value'       => $value_type,
 				'stranded'    => $strand_sense,
 		);
@@ -967,7 +775,6 @@ sub map_relative_data_for_regions {
 		
 		# record the scores
 		record_scores($row, \%relative_scores);
-		
 	}
 }
 
@@ -975,95 +782,33 @@ sub map_relative_data_for_regions {
 
 sub map_relative_long_data_for_regions {
 	
-	# Get the feature indices
-	my (
-		$chr_index, 
-		$start_index, 
-		$stop_index, 
-		$strand_index
-	) = @_;
-	
-	
-	### Collect the data
-	for my $row (1..$main_data_ref->{'last_row'}) {
-		
-		# collect the given coordinates from the data table
-		my ($fstart, $fstop, $fstrand);
-		if (
-			$data_table_ref->[$row][$start_index] <= 
-			$data_table_ref->[$row][$stop_index]
-		) {
-			# proper orientation
-			$fstart = $data_table_ref->[$row][$start_index];
-			$fstop  = $data_table_ref->[$row][$stop_index];
-			
-			# set the strand if not defined
-			unless (defined $strand_index) {
-				$fstrand = 1;
-			}
-		}
-		else {
-			# not a proper orientation of values
-			# assume the user really meant this
-			$fstart = $data_table_ref->[$row][$stop_index];
-			$fstop  = $data_table_ref->[$row][$start_index];
-			
-			# set the strand if not defined
-			unless (defined $strand_index) {
-				$fstrand = -1;
-			}
-		}
-			
-		# determine strand
-		if (defined $strand_index) {
-			# the strand in the data table could be any one of several 
-			# characters denoting strand
-			if ($data_table_ref->[$row][$strand_index] =~ m/^[1|f|w|\+]/i) {
-				$fstrand = 1;
-			}
-			elsif ($data_table_ref->[$row][$strand_index] =~ m/^[r|c|\-]/i) {
-				$fstrand = -1;
-			}
-			else {
-				$fstrand = 0;
-			}
-		}
-		else {
-			$fstrand = 0 unless defined $fstrand;
-		}
-		
-		
+	# Collect the data
+	my $stream = $Data->row_stream;
+	while (my $row = $stream->next_row) {
 		# Collect the scores for each window
 		collect_long_data_window_scores(
-			$row,
-			$data_table_ref->[$row][$chr_index],
-			$fstart,
-			$fstop,
-			$fstrand,
-		);
+			$row, $row->seq_id, $row->start, $row->end, $row->strand);
 	}
-	
 }
 
 
 
 sub record_scores {
 	
-	# get the collected raw scores
+	# row object and raw scores
 	my ($row, $regionscores) = @_;
 	
-	
 	# assign the scores to the windows in the region
 	for (
 		# we will process each window one at a time
 		# proceed by the column index for each window
 		my $column = $startcolumn; 
-		$column < $main_data_ref->{'number_columns'}; 
+		$column < $Data->number_columns; 
 		$column++
 	) {
 		# get start and stop
-		my $start = $main_data_ref->{$column}{'start'};
-		my $stop = $main_data_ref->{$column}{'stop'};
+		my $start = $Data->metadata($column, 'start');
+		my $stop = $Data->metadata($column, 'stop');
 		
 		# collect a score at each position in the window
 		my @scores;
@@ -1118,12 +863,8 @@ sub record_scores {
 		}
 		
 		# put the value into the data table
-		# we're using a push function instead of explicitly assigning 
-		# a position, since the loop is based on relative genomic 
-		# position rather than 
-		$data_table_ref->[$row][$column] = $winscore;
+		$row->value($column, $winscore);
 	}
-
 }
 
 
@@ -1131,7 +872,7 @@ sub record_scores {
 ## Collecting long data in windows
 sub collect_long_data_window_scores {
 	
-	# passed row index and coordinates
+	# passed row object and coordinates
 	my (
 		$row,
 		$fchromo,
@@ -1147,91 +888,82 @@ sub collect_long_data_window_scores {
 		# 5' end of forward strand
 		$reference = $fstart;
 	}
-	
 	elsif ($fstrand == -1 and $position == 5) {
 		# 5' end of reverse strand
 		$reference = $fstop;
 	}
-	
 	elsif ($fstrand >= 0 and $position == 3) {
 		# 3' end of forward strand
 		$reference = $fstop;
 	}
-	
 	elsif ($fstrand == -1 and $position == 3) {
 		# 3' end of reverse strand
 		$reference = $fstart;
 	}
-	
 	elsif ($position == 4) {
 		# midpoint regardless of strand
-		$reference = int( ( ($fstop - $fstart) / 2) + 0.5);
+		$reference = int( ( ($fstop + $fstart) / 2) + 0.5);
 	}
-	
 	else {
 		# something happened
 		die " programming error!? feature " . 
 			" at data row $row\n";
 	}
 	
-	# collect the data
-	# we will be using the get_chromo_region_score() to collect data
-	# for every window 
+	# collect the data for every window 
 	for (
 		my $column = $startcolumn; 
-		$column < $main_data_ref->{'number_columns'}; 
+		$column < $Data->number_columns; 
 		$column++
 	) {
-		$data_table_ref->[$row][$column] = get_chromo_region_score(
+		# we must modify the start and stop position with the adjustments
+		# recorded in the current column metadata
+		my $score = $row->get_score(
 			'db'          => $ddb,
 			'dataset'     => $dataset,
 			'chromo'      => $fchromo,
 			'start'       => $fstrand >= 0 ? 
-								$reference + $main_data_ref->{$column}{'start'} :
-								$reference - $main_data_ref->{$column}{'start'},
+								$reference + $Data->metadata($column, 'start') :
+								$reference - $Data->metadata($column, 'start'),
 			'stop'        => $fstrand >= 0 ? 
-								$reference + $main_data_ref->{$column}{'stop'} : 
-								$reference - $main_data_ref->{$column}{'stop'},
+								$reference + $Data->metadata($column, 'stop') : 
+								$reference - $Data->metdata($column, 'stop'),
 			'strand'      => $fstrand,
 			'method'      => $method,
 			'value'       => $value_type,
 			'stranded'    => $strand_sense,
 			'log'         => $log,
 		);
+		$row->value($column, $score);
 	}
 }
 
 
 
-
-
-
 ## Interpolate the '.' values with the mean of the neighbors
 sub go_interpolate_values {
 	
 	# determine counts
-	my $lastwindow = $main_data_ref->{'number_columns'} - 1; 
+	my $lastwindow = $Data->number_columns - 1; 
 		# lastwindow is the index of the last column
 	
 	# walk through each data line and then each window
-	for my $row (1..$main_data_ref->{'last_row'}) {
+	my $stream = $Data->row_stream;
+	while (my $row = $stream->next_row) {
 		my $col = $startcolumn + 1;
 		while ($col < $lastwindow) {
 			# walk through the windows of a data row
 			# skipping the very first and last windows (columns)
 			# we will look for null values
 			# if one is found, interpolate from neighbors
-			if (
-				$data_table_ref->[$row][$col] eq '.' and 
-				$data_table_ref->[$row][$col - 1] ne '.'
-			) {
+			if ($row->value($col) eq '.' and $row->value($col - 1) ne '.') {
 				# we will interpolate the value from the neighbors
 				# first, need to check that the neighbors have values
 				
 				# find the next real value
 				my $next_i;
 				for (my $i = $col + 1; $col <= $lastwindow; $i++) {
-					if ($data_table_ref->[$row][$i] ne '.') {
+					if ($row->value($i) ne '.') {
 						$next_i = $i;
 						last;
 					}
@@ -1239,13 +971,13 @@ sub go_interpolate_values {
 				next unless defined $next_i;
 				
 				# determine fractional value
-				my $initial = $data_table_ref->[$row][$col - 1];
-				my $fraction = ($data_table_ref->[$row][$next_i] - $initial) / 
+				my $initial = $row->value($col - 1);
+				my $fraction = ($row->value($next_i) - $initial) / 
 					($next_i - $col + 1);
 				
 				# apply fractional values
 				for (my $n = $col; $n < $next_i; $n++) {
-					$data_table_ref->[$row][$n] = $initial + ($fraction * ($n - $col + 1));
+					$row->value($n, $initial + ($fraction * ($n - $col + 1)) );
 				}
 				
 				# jump ahead
@@ -1267,18 +999,14 @@ sub null_to_zeros {
 	# interpolation a lot easier without having to worry about zero
 	
 	# walk through each data line and then each window
-	for my $row (1..$main_data_ref->{'last_row'}) {
-		
-		for (
-			my $col = $startcolumn; 
-			$col < $main_data_ref->{'number_columns'};
-			$col++
-		) {
-			if ($data_table_ref->[$row][$col] eq '.') {
-				$data_table_ref->[$row][$col] = 0;
+	$Data->iterate( sub {
+		my $row = shift;
+		for (my $c = $startcolumn; $c < $Data->number_columns; $c++) {
+			if ($row->value($c) eq '.') {
+				$row->value($c, 0);
 			}
 		}
-	}
+	} );
 }
 
 
@@ -1425,8 +1153,9 @@ Indicate whether features of the same type should be avoided when
 calculating values in a window. Each window is checked for 
 overlapping features of the same type; if the window does overlap 
 another feature of the same type, no value is reported for the 
-window. The default is false (return all values regardless of 
-overlap).
+window. This option requires using named database features and must 
+include a feature GFF type column. The default is false (return all 
+values regardless of overlap).
 
 =item --long
 
