@@ -7,15 +7,15 @@ use Getopt::Long;
 use Pod::Usage;
 use File::Spec;
 use Statistics::Descriptive;
-use Bio::ToolBox::data_helper qw(parse_list);
-use Bio::ToolBox::file_helper qw(load_tim_data_file);
+use Bio::ToolBox::Data;
+use Bio::ToolBox::utility;
 my $gd_ok;
 eval {
 	require GD::Graph::lines; 
 	require GD::Graph::bars; 
 	$gd_ok = 1;
 };
-my $VERSION = '1.15';
+my $VERSION = '1.20';
 
 print "\n This script will plot histograms of value frequencies\n\n";
 
@@ -86,7 +86,10 @@ if ($print_version) {
 
 ### Check required and default values
 unless ($gd_ok) {
-	die "Module GD::Graph must be installed to run this script.\n";
+	die <<GD_WARNING
+Modules GD and GD::Graph failed to load, either because they are not installed or they 
+are missing an external dependency (libgd). Please install these to run this script.
+GD_WARNING
 }
 
 unless ($infile) {
@@ -148,28 +151,23 @@ unless (defined $y_ticks) {
 
 ### Load the file
 print " Loading data from file $infile....\n";
-my $main_data_ref = load_tim_data_file($infile);
-unless ($main_data_ref) {
+my $Data = Bio::ToolBox::Data->new(file => $infile) or
 	die " No data loaded!\n";
-}
-my $data_table_ref = $main_data_ref->{'data_table'};
 
 # load the dataset names into hashes
 my (%dataset_by_name, %dataset_by_id); # hashes for name and id
-for (my $i = 0; $i < $main_data_ref->{'number_columns'}; $i++) {
-	my $name = $main_data_ref->{$i}{'name'};
-	if (
-		# check column header names for gene or window attribute information
-		$name =~ /^(?:name|id|alias|chromosome|start|stop|end|strand|type|class)$/i
-	) { 
-		# skip on to the next header
-		next; 
-	} 
-	else { 
-		# record the data set name
-		$dataset_by_id{$i} = $name;
-		$dataset_by_name{$name} = $i;
-	}
+my $i = 0;
+foreach my $name ($Data->list_columns) {
+	# check column header names for gene or window attribute information
+	# these won't be used for graph generation, so we'll skip them
+	next if $name =~ /^(?:name|id|class|type|alias|probe|chr|
+		chromo|chromosome|seq|sequence|refseq|contig|scaffold|start|stop|end|mid|
+		midpoint|strand|primary_id)$/xi;
+	
+	# record the data set name
+	$dataset_by_id{$i} = $name;
+	$dataset_by_name{$name} = $i;
+	$i++;
 }	
 
 # determine the bins for the frequency distribution
@@ -181,7 +179,9 @@ for my $i (1 .. $binnumber) {
 
 # Prepare output directory
 unless ($directory) {
-	$directory = $main_data_ref->{'path'} . $main_data_ref->{'basename'} . '_graphs';
+	# directory may be specified from a command line argument, otherwise
+	# generate default directory from input file name
+	$directory = $Data->path . $Data->basename . '_graphs';
 }
 unless (-e "$directory") {
 	mkdir $directory or die "Can't create directory $directory\n";
@@ -232,8 +232,8 @@ sub graph_all_datasets {
 sub graph_designated_datasets {
 	my @columns = @_;
 	foreach (@columns) {
-		# we're plotting two datasets
 		if (/&/) { 
+			# we're plotting two datasets
 			my ($one, $two) = split /&/;
 			if ( 
 				(exists $dataset_by_id{$one}) and 
@@ -314,8 +314,9 @@ sub graph_one {
 	print " Preparing graph for $name....\n";
 	my $index = $dataset_by_name{$name};
 	my @values;
-	for my $i (1..$main_data_ref->{'last_row'}) { # walk through the data file
-		my $y = $data_table_ref->[$i][$index];
+	$Data->iterate( sub { 
+		my $row = shift;
+		my $y = $row->value($index);
 		# only take numerical data
 		# must have a numeric value from both datasets, otherwise skip
 		unless (
@@ -325,7 +326,7 @@ sub graph_one {
 		) {
 			push @values, $y; # put into the values array
 		}
-	}
+	} );
 	#print "  found " . scalar @values . " useable values\n";
 	
 	# Determine the data frequency
@@ -339,15 +340,14 @@ sub graph_one {
 	}
 	my $string = '%.' . $x_format . 'f';
 	my @xlabels = map { sprintf $string, $_ } @bins;
-	my @data = ( [@xlabels], [@yvalue] );
 	
 	# Now (finally) prepare the graph
-	my $title = "Distribution of $out $name";
+	my $title = "$out $name";
 	if ($lines) {
-		graph_this_as_lines($name, undef, $title, \@data);
+		graph_this_as_lines($name, undef, $title, [ \@xlabels, \@yvalue ]);
 	} 
 	else {
-		graph_this_as_bars($name, undef, $title, \@data);
+		graph_this_as_bars($name, undef, $title, [ \@xlabels, \@yvalue ]);
 	}
 }	
 
@@ -363,27 +363,27 @@ sub graph_two {
 	my $index1 = $dataset_by_name{$name1};
 	my $index2 = $dataset_by_name{$name2};
 	my (@values1, @values2);
-	for my $i (1..$main_data_ref->{'last_row'}) { 
-		# walk through the data file
-		my $value1 = $data_table_ref->[$i][$index1];
-		my $value2 = $data_table_ref->[$i][$index2];
+	$Data->iterate( sub { 
+		my $row = shift;
+		my $v1 = $row->value($index1);
+		my $v2 = $row->value($index2);
 		# only take numerical data
 		# must have a numeric value from both datasets, otherwise skip
 		unless (
-			$value1 eq '.' or
-			$value1 < $start or
-			$value1 > $max
+			$v1 eq '.' or
+			$v1 < $start or
+			$v1 > $max
 		) {
-			push @values1, $value1; # put into the values array
+			push @values1, $v1;
 		}
 		unless (
-			$value2 eq '.' or
-			$value2 < $start or
-			$value2 > $max
+			$v2 eq '.' or
+			$v2 < $start or
+			$v2 > $max
 		) {
-			push @values2, $value2; # put into the values array
+			push @values2, $v2;
 		}
-	}
+	} );
 	
 	# Determine the data frequency
 	# we have to first determine which dataset has the biggest max value
@@ -406,15 +406,14 @@ sub graph_two {
 	
 	my $string = '%.' . $x_format . 'f';
 	my @xlabels = map { sprintf $string, $_ } @bins;
-	my @data = ( \@xlabels, \@yvalue1, \@yvalue2 );
 	
 	# Now (finally) prepare the graph
-	my $title = "Distributions of $out $name1 & $name2";
+	my $title = "$out $name1 & $name2";
 	if ($lines) {
-		graph_this_as_lines($name1, $name2, $title, \@data);
+		graph_this_as_lines($name1, $name2, $title, [ \@xlabels, \@yvalue1, \@yvalue2 ]);
 	} 
 	else {
-		graph_this_as_bars($name1, $name2, $title, \@data);
+		graph_this_as_bars($name1, $name2, $title, [ \@xlabels, \@yvalue1, \@yvalue2 ]);
 	}
 }
 
